<?php
/* 
+----------------------------------------------------------------------+
| (c) Copyright IBM Corporation 2005.                                  |
| All Rights Reserved.                                                 |
+----------------------------------------------------------------------+
|                                                                      |
| Licensed under the Apache License, Version 2.0 (the "License"); you  |
| may not use this file except in compliance with the License. You may |
| obtain a copy of the License at                                      |
| http://www.apache.org/licenses/LICENSE-2.0                           |
|                                                                      |
| Unless required by applicable law or agreed to in writing, software  |
| distributed under the License is distributed on an "AS IS" BASIS,    |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or      |
| implied. See the License for the specific language governing         |
| permissions and limitations under the License.                       |
+----------------------------------------------------------------------+
| Author: Simon Laws                                                   |
+----------------------------------------------------------------------+
$Id: 1c-CRUDDuffSQL.php,v 1.2 2007/02/07 11:26:06 cem Exp $
*/

echo "executing scenario one-company-create/retrieve/update/Collision/delete\n";

require_once 'SDO/DAS/Relational.php';
require_once 'company_metadata.inc.php';


/*************************************************************************************
* Use SDO to perform create, retrieve, update, and delete operations on a row of the company table.
* Here invalid SQL is used in a query to cause any invalid SQL exception to be thrown
*
* See companydb_mysql.sql and companydb_db2.sql for examples of defining the database 
*
*************************************************************************************/

/*************************************************************************************
 * Empty out the company table
 *************************************************************************************/
$dbh = new PDO(PDO_DSN,DATABASE_USER,DATABASE_PASSWORD);
$count = $dbh->exec('DELETE FROM company;');
echo "Emptied out the company table with DELETE FROM company\n";


/*************************************************************************************
* Create the root data object then a single company object underneath it.
* Set the company name to 'Acme'.
*************************************************************************************/
$das = new SDO_DAS_Relational ($database_metadata,'company',$SDO_reference_metadata);
$dbh = new PDO(PDO_DSN,DATABASE_USER,DATABASE_PASSWORD);

$root 		= $das  -> createRootDataObject();
$company 	= $root -> createDataObject('company');
$company -> name = "Acme";
$company -> employee_of_the_month = null;

$das -> applyChanges($dbh, $root);
echo "Created a company with name Acme and wrote it to the database\n";

/*************************************************************************************
 * Find it again (with its autogenerated id this time).
 * Here invalid XML is used to force an invalid SQL exception
 *************************************************************************************/
$das = new SDO_DAS_Relational ($database_metadata,'company',$SDO_reference_metadata);
$dbh = new PDO(PDO_DSN,DATABASE_USER,DATABASE_PASSWORD);

try {
  $name = 'Acme';
  $pdo_stmt = $dbh->prepare('select xname, id, employee_of_the_month from company where name=?');
  $root = $das->executePreparedQuery($dbh, $pdo_stmt ,array($name), array('company.name', 'company.id','company.employee_of_the_month'));
  echo "This test should have thrown an exception indicating invalid SQL but nothing happened\n";
} catch (SDO_Exception $e) {
    echo "Expected exception indicating invalid SQL got... \n\n";
    print "getMessage(): ".$e->getMessage()."\n\n";
}


/*************************************************************************************
* Find it again under its old name, and delete it. Use valid SQL this time
* Just for a change, re-use the PDO database handle and prepared statement.
*************************************************************************************/

$name = 'Acme';
 $pdo_stmt = $dbh->prepare('select name, id, employee_of_the_month from company where name=?');
$root = $das->executePreparedQuery($dbh, $pdo_stmt ,array($name), array('company.name', 'company.id','company.employee_of_the_month'));
$company3 = $root['company'][0];
echo "Looked for Acme and found company with name = " . $company3->name . " and id " . $company3->id . "\n";
assert($company3->name == 'Acme');
unset($root['company'][0]); // do not make the mistake of doing unset($company3) - this will just destroy the $company3 variable
$das -> applyChanges($dbh, $root);
echo "Deleted the company and wrote the changes back to the database\n";

/*************************************************************************************
* Check the row is really gone
*************************************************************************************/
$dbh = new PDO(PDO_DSN,DATABASE_USER,DATABASE_PASSWORD);

foreach($dbh->query('select * from company') as $row) {
  assert(false); // There had better be no such rows. 
}

echo "Checked that the table was truly empty with SELECT * FROM company. Nothing was found so the delete was successful.\n";

?>
